#! /usr/bin/python

import smtplib
import os
import sys
import time
import socket

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.charset import Charset
from email.header import Header
import email.Charset
# http://bytes.com/topic/python/answers/27012-problems-creating-mail-content-using-email-mimetext-non-ascii-encoding
email.Charset.add_charset('utf-8', email.Charset.SHORTEST, None, None)
import config

class NoMoreAttemptsException(Exception): pass

class Message(object):
    '''Represent a message to send'''

    def __init__(self, recipient, subject, plain, html=None, sender=None, encoding=None):
        '''Create a new message.

        recipient -- the recipient
        subject -- the subject
        plain -- the plain message
        html -- the html message (default is None)
        sender -- the sender of the email (if None -as the default-
                  sender is set to the config value.'''

        if not sender:
            sender = config.conf['mailer']['sender']
        self.sender = sender
        self.recipient = recipient
        self.subject = subject 
        self.encoding = encoding or config.conf['mailer']['encoding']

        # XXX: mmm
        if isinstance(plain, unicode) and self.encoding.lower() == 'utf-8':
            plain = plain.encode('utf-8')
        if isinstance(html, unicode) and self.encoding.lower() == 'utf-8':
            html = html.encode('utf-8')

        self.plain = plain
        self.html = html
    
    def as_string(self):
        # Create message container - the correct MIME type is multipart/alternative.
        msg = MIMEMultipart('alternative')
        msg['Subject'] = Header(self.subject, self.encoding)
        # FIXME: sender and recipient must be decoded like the subject?
        msg['From'] = self.sender
        msg['To'] = self.recipient

        plain_part = MIMEText(self.plain, 'plain', self.encoding)
        msg.attach(plain_part)

        if self.html:
            html_part = MIMEText(self.html, 'html', self.encoding)
            msg.attach(html_part)

        return msg.as_string()

class Mailer(object):
    '''Send a Message'''

    def __init__(self, batch=False):
        '''Constructor.
        
        Batch is used for auto-connecto/disconnect, if batch is False
        (default) the Mailer disconnect after the .send(), if it is True
        it will not disconnect.
        
        Batch mode avoids to disconnect after every single sending.'''
        self.connected = False
        self.batch = batch

    def _connect(self):
        '''Try to connect to the SMTP.'''

        if self.connected:
            return

        smtpserver = config.conf['mailer']['host']
        smtpport = config.conf['mailer']['port']
        username = config.conf['mailer']['username']
        password = config.conf['mailer']['password']
        timeout = config.conf['mailer']['timeout']

        try:
            self.connection = smtplib.SMTP(smtpserver, smtpport, timeout=timeout)
            if username and password:
                config.log.debug('using username and password')
                self.connection.login(username, password)
            config.log.debug('connected to %s' % smtpserver)
            self.connected = True

        except smtplib.SMTPHeloError, e:
            config.log.critical('Helo Error %s', e)
            raise
        except smtplib.SMTPAuthenticationError, e:
            config.log.critical('Wrong username and/or password')
            raise
        except smtplib.SMTPConnectError, e:
            config.log.critical('Connection error: %s' % e)
            raise
        except smtplib.SMTPException, e:
            config.log.critical('General Error: %s' % e)
            raise
        except socket.timeout, e:
            config.log.critical('Connection timeout')
            raise

    def _disconnect(self):
        '''Try to disconnect to the SMTP.'''

        smtpserver = config.conf['mailer']['host']
        self.connection.quit()
        config.log.debug('disconnected to %s' % smtpserver)
        self.connected = False

    def _send(self, message):
        try:
            config.log.debug('trying to email %s' % message.recipient)

            self.connection.sendmail(
                message.sender,
                message.recipient,
                message.as_string())

            config.log.info('message sent to %s' % message.recipient)
        except smtplib.SMTPServerDisconnected, e:
            config.log.warning('SMTP server disconnected: %s' % e)
            self.connected = False
            raise
        except smtplib.SMTPSenderRefused, e:
            config.log.critical('SMTP server refused the sender: %s' % e)
            raise
        except smtplib.SMTPRecipientsRefused, e:
            config.log.warning('SMTP server refused the recipient: %s' % e)
            raise
        except smtplib.SMTPDataError, e:
            config.log.critical('SMTP data error: %s' % e)
            raise
        except IOError, e:
            config.log.warning('I/O error: %s' % e)
            self.connected = False
            raise

    def send(self, message):
        '''Send a message.
        
        If the send fails*, this method will retry a fixed number of times
        (configured via the mailer.attempts value); if also the last attempt
        fails, 'send' raises a NoMoreAttemptsException.

        You should catch smtplib.SMTPRecipientsRefused and blacklist the refused
        recipients.
        
        * "fail" means 1) the server disconnected 2) an IOError occured
          3) a timeout occurred.'''

        left_attempts = config.conf['mailer']['attempts']

        while True:
            left_attempts -= 1
            try:
                self._connect()
                self._send(message)
                break
            except (smtplib.SMTPServerDisconnected, IOError, socket.timeout):
                config.log.debug('Email %s has failed, %s attempts left' %\
                    (left_attempts, message.recipient))
                if not left_attempts:
                    raise NoMoreAttemptsException()

        if not self.batch:
            self._disconnect()

def send(message):
    m = Mailer()
    return m.send(message)




import csv

class UnicodeCSVDictReader(csv.DictReader):
    '''Used to convert each line to the specified encoding'''

    def __init__(self, f, encoding='utf-8', *args, **kw):
        self.encoding = encoding
        csv.DictReader.__init__(self, f, *args, **kw)

    def next(self):
        line = csv.DictReader.next(self)
        line = dict(zip(
            line.keys(),
            map(lambda v: v.decode(self.encoding).trim(), line.values())))
        return line




#!/usr/bin/env python

from setuptools import setup

setup(
    name='Spamalot',
    version='0.0.3',
    description='Python simple mailer',
    author='Alberto Granzotto (vrde)',
    author_email='agranzot@gmail.com',
    url='http://hg.tastybra.in/public/spamalot',

    packages=['spamalot'],

    package_data = {
        'spamalot': ['conf/*']
    },

    install_requires=[
        'ConfigObj',
        'setuptools'
    ]
)




from configobj  import ConfigObj, flatten_errors
from validate import Validator

import logging
import logging.config

import pkg_resources

conf = None
log = None

# sorry for that...
class Dummy(object):
    def foo(self, *args, **kw):
        pass
    def __getattr__(self, name):
        return self.foo

def _trim_config(configuration, prefix):
    # ty sqlalchemy :P
    options = dict((key[len(prefix):], configuration[key])
                   for key in configuration
                   if key.startswith(prefix))
    return options

def init_from_config(configuration, prefix='spamalot.'):
    mailer = _trim_config(configuration, '%s%s' % (prefix, 'mailer.'))
    logger = _trim_config(configuration, '%s%s' % (prefix, 'logger.'))
    options = {'mailer': mailer, 'logger': logger}
    init_mailer(options)

def init_mailer(configfile=None):
    """Feed the mailer with the configuration file"""
    
    global conf, log

    configspec = pkg_resources.resource_filename('spamalot',
        'conf/configspec.ini')
    logconfig = pkg_resources.resource_filename('spamalot',
        'conf/logging.ini')
    
    conf = ConfigObj(configfile, configspec=configspec)
    validator = Validator()
    results = conf.validate(validator)

    if results != True:
        for (section_list, key, _) in flatten_errors(conf, results):
            if key is not None:
                print 'The "%s" key in the section "%s" failed validation' %\
                    (key, ', '.join(section_list))
            else:
                print 'The following section was missing' % ', '.join(section_list)
        exit(0)

    if conf['logging']['outfile']:
        defaults = {
            'custom_outfile': conf['logging']['outfile'],
            'custom_mode': conf['logging']['mode'],
            'custom_level': conf['logging']['level']
        }

        logging.config.fileConfig(logconfig, defaults)
#    else:
#        log = Dummy()
    log = logging.getLogger('spamalot')
    log.info('spamalot initialised')




#!/usr/bin/python

import smtplib
import codecs
import os
import sys
import time
import logging
from string import Template

import config
import spamalot
from spamalot import utils


class Spammer(object):
    '''
    attributes:
        recipients  -- a dict with all the data of the recipients;
        all_addrs   -- the set of all addresses loaded;
        left_addrs  -- the set of the left addresses to email;
        sent_addrs  -- the set of addresses e-mailed;
        black_addrs -- the set of blacklisted addresses;
    '''

    def __init__(self, mailer):
        self.log = logging.getLogger('spammer')
        self.file_encoding = config.conf['spammer']['file_encoding']
        self._parse_message()
        self._parse_recipients()
        self._parse_sent()
        self._parse_blacklist()
        self.mailer = mailer

    @property
    def left_addrs(self):
        return self.all_addrs - (self.sent_addrs | self.black_addrs)

    def _prepare_message(self, recipient):
        message = spamalot.Message(
            recipient['email'],
            # FIXME: check encoding
            config.conf['spammer']['subject'],
            self.plain_email_template.substitute(recipient),
            self.html_email_template.substitute(recipient),
        )
        return message

    def _parse_message(self):
        '''Parse the text and html messages to send.'''

        plain = config.conf['spammer']['plain_message']
        html = config.conf['spammer']['html_message']

        self.plain_email_template = Template(codecs.open(
            plain, 'r', self.file_encoding).read())

        self.html_email_template = Template(codecs.open(
            html, 'r', self.file_encoding).read())

        self.log.debug('loaded plain and html messages')

    def _parse_recipients(self):
        '''Parse the csv file.'''

        filename = config.conf['spammer']['recipients']
        reader = utils.UnicodeCSVDictReader(
            open(filename, 'rb'), self.file_encoding)

        self.recipients = []
        self.all_addrs = set()

        for line in reader:
            self.recipients.append(line)
            self.all_addrs.add(line['email'])

        self.log.debug('loaded %d recipients' % len(self.recipients))

    def _parse_sent(self):
        '''Parse the list of sent email.'''

        self.sent_addrs = set()
        try:
            for line in open('spammer-email-sent').readlines():
                if not line: continue
                self.sent_addrs.add(line.strip())
            self.log.debug('loaded sent list')
        except:
            self.log.debug('sent list not found')

    def _parse_blacklist(self):
        '''Parse the blacklist of email.'''

        self.black_addrs = set()
        try:
            for line in open('spammer-email-blacklist').readlines():
                self.black_addrs.add(line.strip())
            self.log.debug('loaded blacklist')
        except:
            self.log.debug('blacklist not found')

    def _single_send(self, recipient, msg):
        try:
            if config.conf['spammer']['really']:
                self.mailer.send(msg)
            self.sent_addrs.add(recipient['email'])
            f = open('spammer-email-sent', 'a')
            f.write('%s\n' % recipient['email'])
            f.close()

        except spamalot.NoMoreAttemptsException, e:
            self.log.info('Too much errors for %s, will try later' % recipient['email'])

        except smtplib.SMTPRecipientsRefused, e:
            self.log.warning('SMTP server refused the recipient: %s' % e)
            self.black_addrs.add(recipient['email'])
            f = open('spammer-email-blacklist', 'a')
            f.write('%s\n' % recipient['email'])
            f.close()

    def _send(self, to_addrs):
        delay = config.conf['spammer']['delay']

        for recipient in self.recipients:
            if recipient['email'] in to_addrs:
                msg = self._prepare_message(recipient)
                self._single_send(recipient, msg)
                time.sleep(delay)
            else:
                self.log.debug('skipping %s' % recipient['email'])

    def send(self):
        self.log.info('spamming started, found %d recipients' % len(self.left_addrs))

        attempt = 1
        tot_attemps = 3
        while attempt <= tot_attemps and self.left_addrs:
            self.log.debug('start attempt #%d' % attempt)
            self._send(self.left_addrs)
            attempt -= 1

        if not self.left_addrs:
            self.log.info("spamming finished, everything fine capt'n!")
        else:
            self.log.warning('there are %d left email to send!' % len(self.left_addrs))

if __name__ == '__main__':
    spamalot.init_mailer('spam.ini')
    mailer = spamalot.Mailer(batch=True)
    spammer = Spammer(mailer)
    spammer.send()

